import os

import pytest
from pymongo import MongoClient
from pymongo.database import Database

from pytest_scenarios.scenario import ScenarioBuilder
from pytest_scenarios.template_loader import load_templates_from_path


def _get_option(request: pytest.FixtureRequest, name: str, default=None):
    value = request.config.getoption(
        f"--{name}", default=os.environ.get(name.upper().replace("-", "_"), default)
    )
    print(f"Using option {name}={value}")
    return value


@pytest.fixture(scope="session")
def templates_path(request: pytest.FixtureRequest):
    return _get_option(request, "templates-path", default="tests/templates")


@pytest.fixture(scope="session")
def mongo_client(request: pytest.FixtureRequest):
    db_url = _get_option(
        request, "db-url", default="mongodb://127.0.0.1:27017/?directConnection=true"
    )
    with MongoClient(db_url) as client:
        yield client


@pytest.fixture(scope="session")
def db(request: pytest.FixtureRequest, mongo_client: MongoClient):
    db_name = _get_option(request, "db-name", default="test_db")
    yield mongo_client[db_name]


@pytest.fixture(scope="session")
def scenario_builder(db: Database, templates_path: str) -> ScenarioBuilder:
    templates = load_templates_from_path(templates_path)
    return ScenarioBuilder(db, templates)


@pytest.fixture(scope="function", autouse=True)
def cleanup_database(scenario_builder: ScenarioBuilder):
    """Clear all collections in the database before each test function."""
    scenario_builder.cleanup_collections()



"""
A utility to dynamically load template dictionaries from modules in a given package.
"""

import importlib
import os
from typing import Any


def load_templates_from_path(path: str) -> dict[str, Any]:
    """
    Loads all TEMPLATE dictionaries from Python files in the given directory.

    Args:
        path: Filesystem path to the directory containing template modules.

    Returns:
        A dictionary mapping filename (without .py) to the TEMPLATE dict in each file.
    """
    templates = {}
    abs_path = os.path.abspath(path)
    for filename in os.listdir(abs_path):
        if filename.endswith(".py") and filename != "__init__.py":
            module_name = filename[:-3]
            file_path = os.path.join(abs_path, filename)
            spec = importlib.util.spec_from_file_location(module_name, file_path)
            if not spec or not spec.loader:
                continue
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)
            if hasattr(module, "TEMPLATE"):
                templates[module_name] = module.TEMPLATE
    return templates



from collections.abc import Iterable

from bson import ObjectId
from pymongo.database import Database


class ScenarioBuilder:
    def __init__(self, db: Database, templates: dict[str, dict]):
        """Initialize the ScenarioBuilder with a MongoDB database and templates.
        Args:
            db: The MongoDB database instance.
            templates: A dictionary of templates to be used as blueprints for creating documents.
            The keys are collection names and the values are the template documents.
            We also create the collections in the database.
        """
        self._db = db
        self._templates = templates
        self._init_collections()

    def create(
        self, scenario: dict[str, Iterable[dict]], add_scenario_id=False
    ) -> Iterable[tuple[str, list[ObjectId]]]:
        """Create a scenario with the given steps.
        The scenario is a dictionary where keys are collection names
        and values are iterables of documents to insert into those collections."""
        scenario_id = ObjectId()
        scenario_doc = {"scenario_id": scenario_id} if add_scenario_id else {}
        for collection_name, docs in scenario.items():
            collection = self._db[collection_name]
            template = self._templates.get(collection_name, {})
            docs_to_insert = [template | doc | scenario_doc for doc in docs]
            result = collection.insert_many(
                docs_to_insert, comment=f"ScenarioBuilder {scenario_id}"
            )
            if len(result.inserted_ids) != len(docs_to_insert):
                raise ValueError("Failed to insert all documents")

            yield collection_name, result.inserted_ids

    def _init_collections(self) -> Iterable[dict]:
        """Register templates in the database.
        The templates is a dictionary where keys are collection names
        and values are iterables of documents to insert into those collections."""
        for collection_name in self._templates:
            self._db.create_collection(collection_name, check_exists=False)

    @property
    def collections(self) -> Iterable[str]:
        """Return the collection names managed by this ScenarioBuilder."""
        return self._templates.keys()

    def cleanup_collections(self):
        """Clear all collections managed by this ScenarioBuilder."""
        for name in self.collections:
            self._db[name].delete_many({})
